#include <amxmodx>
#include <amxmisc>
#include <cstrike>
#include <hamsandwich>
#include <fakemeta_util>
#include <zombieplague>

#define PLUGIN "Holy Bomb"
#define VERSION "1.0"
#define AUTHOR "Arwel"

#define TASKID_BURN 3462
#define TASKID_REMOVE_BURN 3463

new const ViewModel[]="models/holy_bomb/v_holybomb.mdl"
new const PlayerModel[]="models/holy_bomb/p_holybomb.mdl"
new const WorldModel[]="models/holy_bomb/w_holybomb.mdl"

new const explode_sprite[]="sprites/holy_bomb/holybomb_exp.spr"
new const burn_sprite[]="sprites/holy_bomb/holybomb_burn.spr"

new const ExplodeSounds[][]=
{
	"weapons/holywater_deploy.wav",
	"weapons/holywater_pinpul.wav",
	"weapons/holywater_explosion.wav"
}

new const NADE_TYPE_HOLYBOMB = 7000


new const item_cost = 20
new const item_name[] = "Holy Bomb"

new item_id

new has_bomb[33], in_burn[33]

new cvar_flaregrenades

new pcvar_radius, pcvar_damage
new pcvar_burn_damage, pcvar_burn_num, pcvar_burn_period

new g_exploSpr

public plugin_init() 
{
	register_plugin(PLUGIN, VERSION, AUTHOR)
	
	RegisterHam(Ham_Touch, "grenade", "fw_TouchGrenade", 1)
	RegisterHam(Ham_Killed, "player", "fw_PlayerKilled")
	RegisterHam(Ham_RemovePlayerItem, "player", "fw_DropGrenade")
	
	register_forward(FM_SetModel, "fw_SetModel")
	
	register_event("HLTV", "EventRoundStart", "a", "1=0", "2=0")
	register_event("CurWeapon", "Event_CurWeapon", "be", "1=1")
	
	register_clcmd("weapon_holybomb_by_arwel", "anull_model")
	
	pcvar_radius=register_cvar("holy_bomb_radius", "170")
	pcvar_damage=register_cvar("holy_bomb_damage", "444")
	
	pcvar_burn_damage=register_cvar("holy_bomb_burn_damage","55")
	pcvar_burn_num=register_cvar("holy_bomb_burns_num", "8")
	pcvar_burn_period=register_cvar("holy_bomb_burns_period", "1")
	
	cvar_flaregrenades = get_cvar_num("zp_flare_grenades")
	
	item_id = zp_register_extra_item(item_name,item_cost,ZP_TEAM_HUMAN)	
}

public plugin_cfg()
{
	new ConfigDir[128]
	get_configsdir(ConfigDir, 127)
	server_cmd("exec %s/holy_bomb.cfg",ConfigDir)
	
	pcvar_radius=get_cvar_num("holy_bomb_radius")
	pcvar_damage=get_cvar_num("holy_bomb_damage")
	
	pcvar_burn_damage=get_cvar_num("holy_bomb_burn_damage")
	pcvar_burn_num=get_cvar_num("holy_bomb_burns_num")
	pcvar_burn_period=get_cvar_num("holy_bomb_burns_period")
	
}

public plugin_precache()
{
	g_exploSpr = precache_model(explode_sprite)
	
	precache_model(burn_sprite)
	precache_model(ViewModel)
	precache_model(PlayerModel)
	precache_model(WorldModel)
	
	for(new i; i<=charsmax(ExplodeSounds); i++)
		precache_sound(ExplodeSounds[i])
	
	precache_generic("sprites/weapon_holybomb_by_arwel.txt")
	precache_generic("sprites/holy_bomb/640hud61.spr")
}

public plugin_natives()
{
	register_native("cso_give_holybomb","native_give_holybomb", 1)
	register_native("cso_take_holybomb","native_take_holybomb", 1)	
}

public native_give_holybomb(id)
{
	ham_strip_weapon(id, "weapon_smokegrenade")
	has_bomb[id] = 1	
	
	message_begin( MSG_ONE, get_user_msgid("WeaponList"), .player=id )
	write_string("weapon_holybomb_by_arwel") 
	write_byte(13)
	write_byte(1)
	write_byte(-1)
	write_byte(-1)
	write_byte(3)
	write_byte(3)
	write_byte(CSW_SMOKEGRENADE)
	write_byte(24)
	message_end()
	
	fm_give_item(id,"weapon_smokegrenade")	
}

public native_take_holybomb(id)
{
	ham_strip_weapon(id, "weapon_smokegrenade")
	has_bomb[id] = 0
	
}
	
public client_disconnect(id)
{
	has_bomb[id] = 0
	
	RemoveBurnEntity(id)
}

public zp_extra_item_selected(id, itemid)
{
	if(itemid == item_id)
	{
		native_give_holybomb(id)
	}
}

public zp_user_humanized_pre(id)
{
	
	RemoveBurnEntity(id)
	
}

public fw_DropGrenade(id, ent)
{
	new wEnt
	while((wEnt = engfunc(EngFunc_FindEntityByString,wEnt,"classname","weapon_smokegrenade")) && pev(wEnt,pev_owner) != id){}
	if(wEnt==ent)
	{
		message_begin( MSG_ONE, get_user_msgid("WeaponList"), .player=id )
		write_string("weapon_smokegrenade") 
		write_byte(13)
		write_byte(1)
		write_byte(-1)
		write_byte(-1)
		write_byte(3)
		write_byte(3)
		write_byte(CSW_SMOKEGRENADE)
		write_byte(24)
		message_end()
	}
}

public anull_model(id)
{
	
	client_cmd(id, "weapon_smokegrenade")

}

public EventRoundStart()
{
	
	fm_remove_entity_name("HolyBombFire")
	arrayset(in_burn, -1, 32)
	
}

public Event_CurWeapon(id)
{
	if (!is_user_alive(id))
		return
	
	if(get_user_weapon(id) == CSW_SMOKEGRENADE && has_bomb[id])
	{
		set_pev(id, pev_viewmodel2, ViewModel)	
	}
}

public fw_SetModel(ent, const model[])
{
	
	static Float:dmgtime
	pev(ent, pev_dmgtime, dmgtime)
	
	if (dmgtime == 0.0)
		return
	
	if (equal(model[7], "w_sm", 4))
	{		
		new owner = pev(ent, pev_owner)		
		
		if(!zp_get_user_zombie(owner) && has_bomb[owner]) 
		{
			set_cvar_num("zp_flare_grenades",0)
			fm_set_rendering(ent, kRenderFxGlowShell, 90, 155, 255, kRenderNormal, 3)
			set_pev(ent, pev_dmgtime, get_gametime()+9999999.0)
			set_pev(ent, pev_nextthink, get_gametime()+9999999.0)
			set_pev(ent, pev_flTimeStepSound, NADE_TYPE_HOLYBOMB)
			
			has_bomb[owner]=0
		}
	}
}

public fw_SetModelPost(ent, const model[])
{	
	if(pev(ent, pev_flTimeStepSound) == NADE_TYPE_HOLYBOMB)
		fm_entity_set_model(ent,WorldModel)	
}

public fw_TouchGrenade(ent)
{
	if(pev(ent, pev_flTimeStepSound) == NADE_TYPE_HOLYBOMB)
		make_explode(ent)
}

public make_explode(ent)
{
	static Float:fOrigin[3], iOrigin[3], owner
	
	owner=pev(ent, pev_owner)
	
	pev(ent, pev_origin, fOrigin)
	FVecIVec(fOrigin, iOrigin)
	
	message_begin(MSG_BROADCAST, SVC_TEMPENTITY, {0, 0, 0}, 0)
	write_byte(TE_EXPLOSION)
	write_coord(iOrigin[0])
	write_coord(iOrigin[1])
	write_coord(iOrigin[2]+30)
	write_short(g_exploSpr)
	write_byte(10)
	write_byte(28)
	write_byte(TE_EXPLFLAG_NOSOUND|TE_EXPLFLAG_NOPARTICLES)
	message_end()
	
	message_begin(MSG_BROADCAST, SVC_TEMPENTITY)
	write_byte(TE_DLIGHT) 
	write_coord(iOrigin[0])
	write_coord(iOrigin[1])
	write_coord(iOrigin[2])
	write_byte(10) 
	write_byte(30)
	write_byte(100)
	write_byte(255)
	write_byte(5) 
	write_byte(0) 
	message_end()
	
	emit_sound(ent, CHAN_STATIC, ExplodeSounds[2], VOL_NORM, ATTN_NORM, 0, PITCH_NORM)
	
	 
	static player=FM_NULLENT
	while((player = fm_find_ent_in_sphere(player, fOrigin, float(pcvar_radius)))!=0)
	{
		
		if(pev(player, pev_takedamage)!=DAMAGE_NO&&pev(player, pev_solid)!=SOLID_NOT)
		{
			if( is_user_connected(player))
			{
				if(is_user_alive(player)&&zp_get_user_zombie(player))
				{
					ExecuteHamB( Ham_TakeDamage, player ,owner ,ent, float(pcvar_damage), DMG_BLAST )
					if(!is_user_alive(player))
						log_kill(owner, player, "holy_bomb",0)
					else
						CreateBurnEntity(player, owner)
				}
			}
			else
			{
				
				ExecuteHamB( Ham_TakeDamage, player ,owner ,ent, float(pcvar_damage), DMG_BLAST )
				
			}
		}
	}
	set_cvar_num("zp_flare_grenades",cvar_flaregrenades)
	fm_remove_entity(ent)
}


public CreateBurnEntity(victim, attacker)
{
	if(!is_user_alive(victim)||!is_user_alive(attacker))
		return
	if(in_burn[victim]>0)
		return
		
	new ent = fm_create_entity("cycler_sprite")
	
	set_pev(ent, pev_owner, attacker)
	
	set_pev(ent, pev_model, burn_sprite)
	dllfunc(DLLFunc_Spawn, ent)
	
	set_pev(ent, pev_classname, "HolyBombFire")
	
	fm_entity_set_model(ent, burn_sprite)
	
	fm_set_rendering(ent, kRenderFxNoDissipation, 255, 255, 255, kRenderGlow, 255)
	
	set_pev(ent, pev_framerate, 10.0)
	
	set_pev(ent, pev_solid, SOLID_NOT)
	set_pev(ent, pev_movetype, MOVETYPE_FOLLOW)
	set_pev(ent, pev_aiment, victim)
	
	
	in_burn[victim]=ent
	
	set_task(float(pcvar_burn_period)*pcvar_burn_num, "TaskRemoveBurnEntity", victim+TASKID_REMOVE_BURN)
	
	set_task(float(pcvar_burn_period), "Burn", victim+TASKID_REMOVE_BURN, _,_,"a", pcvar_burn_num)
	
}

public Burn(taskid)
{
	new id = taskid-TASKID_REMOVE_BURN
	
	if(!is_user_alive(id))
		return
		
	if(!pev_valid(in_burn[id]))
		return
	
	new Float:health
	
	pev(id, pev_health, health)
	
	if(health<=float(pcvar_burn_damage))
	{
		remove_task(taskid)
		return
	}
	
	health-=float(pcvar_burn_damage)
	set_pev(id, pev_health, health)
}

public TaskRemoveBurnEntity(taskid)
{
	new id = taskid-TASKID_REMOVE_BURN	
	
	RemoveBurnEntity(id)
}

public RemoveBurnEntity(id)
{
	if(!pev_valid(in_burn[id]))
		return
	
	fm_remove_entity(in_burn[id])
	in_burn[id]=-1
}
public fw_PlayerKilled(id, attacker, shouldgib)
{
	has_bomb[id] = 0
	
	remove_task(id+TASKID_REMOVE_BURN)
	
	RemoveBurnEntity(id)		
}

public log_kill(killer, victim, weapon[],headshot) 
{
	if(is_user_alive(victim))
		return PLUGIN_CONTINUE

	message_begin(MSG_ALL, get_user_msgid("DeathMsg"), {0,0,0}, 0)
	write_byte(killer)
	write_byte(victim)
	write_byte(headshot)
	write_string(weapon)
	message_end()
	
	fm_set_user_frags(killer,get_user_frags(killer) +1)
	message_begin(MSG_ALL, get_user_msgid("ScoreInfo")) 
	write_byte(killer) 
	write_short(get_user_frags(killer)) 
	write_short(get_user_deaths(killer)) 
	write_short(0) // 0 under CS, but what about other mods? 
	write_short(get_user_team(killer)) 
	message_end() 
	message_begin(MSG_ALL, get_user_msgid("ScoreInfo")) 
	write_byte(victim) 
	write_short(get_user_frags(victim)) 
	write_short(get_user_deaths(victim)) 
	write_short(0) // 0 under CS, but what about other mods? 
	write_short(get_user_team(victim)) 
	message_end() 	
	new kname[32], vname[32], kauthid[32], vauthid[32], kteam[10], vteam[10]
	
	get_user_name(killer, kname, 31)
	get_user_team(killer, kteam, 9)
	get_user_authid(killer, kauthid, 31)
	
	get_user_name(victim, vname, 31)
	get_user_team(victim, vteam, 9)
	get_user_authid(victim, vauthid, 31)
	
	log_message("^"%s<%d><%s><%s>^" killed ^"%s<%d><%s><%s>^" with ^"%s^"", 
	kname, get_user_userid(killer), kauthid, kteam, 
	vname, get_user_userid(victim), vauthid, vteam, weapon)
	
	return PLUGIN_CONTINUE
}

stock ham_strip_weapon(id,weapon[])
{
	if(!equal(weapon,"weapon_",7)) return 0;

	new wId = get_weaponid(weapon);
	if(!wId) return 0;

	new wEnt;
	while((wEnt = engfunc(EngFunc_FindEntityByString,wEnt,"classname",weapon)) && pev(wEnt,pev_owner) != id) {}
	if(!wEnt) return 0;
	
	if(get_user_weapon(id) == wId) ExecuteHamB(Ham_Weapon_RetireWeapon,wEnt);

	if(!ExecuteHamB(Ham_RemovePlayerItem,id,wEnt) ) return 0;
	ExecuteHamB(Ham_Item_Kill,wEnt);

	set_pev(id,pev_weapons,pev(id,pev_weapons) & ~(1<<wId));

	return 1;
}